 
% Solves parts B and D. 
 
% April 26, 1999 
% Developed by Ali H. Sayed 
 
close all
clear all
clear; 
clf; 
 
var_v=0.004;   % (25dB SNR)  % variance of noise; CAN BE CHANGED 
%var_v=0.05;    % (14dB SNR)
                 % Since complex-valued, the variance of the  
                 % real and imaginary parts are var_v/2. 
 
 
N=2000;          % number of symbols to be generated 
 
 
s=(sign(randn(1,N))+i*sign(randn(1,N)))/(sqrt(2)); 
                              % generates QPSK data 
y=filter([1 0.5],1,s);        % Filters the data through the channel 
 
y=y+sqrt(var_v/2)*randn(1,N)+sqrt(var_v/2)*i*randn(1,N);  
                              % adds complex noise to it 
 
 
% We form the covariance matrix R_y 
% Its (1,1) entry is kept in terms of the noise 
% variance for generality 
 
Ry=[((5/4)+var_v) 1/2   0 
 
    1/2  ((5/4)+var_v)  1/2 
 
    0   1/2    ((5/4)+var_v)]; 
 
 
Ryinv=inv(Ry); 
 
 
 
%%%% We now plot all symbols in constellation  
%%%% form for both the transmitted and received signals 
  
%%%% We plot the scatter diagrams of 
%%%% both the transmitted and received signals 
 

subplot(221); 
plot(real(s),imag(s),'x');grid; 
title('Transmitted sequence s(i)'); 
 
 
subplot(222); 
plot(real(y),imag(y),'x');grid; 
title('Received sequence y(i)'); 
 

 
for Delta=0:3    % three values of Delta 
 
  % The value of Rxy depends on the choice of Delta 
 
   switch Delta 
 
   case 0, Rxy = [1 0 0]; 
 
   case 1, Rxy = [0.5 1 0]; 
 
   case 2, Rxy = [0 0.5 1]; 
 
   case 3, Rxy = [0 0 0.5]; 
 
   end 
 
    
 
   Ko = Rxy*Ryinv;       % Optimal equalizer 
 
   s_hat=filter(Ko,1,y); % Filter y(i) through equalizer 
                         % to generate hat{s}(i-Delta) 
 
    
    %%%% We now plot in constellation  
    %%%% form the transmitted and recovered signals 
 
    if (Delta == 0) | (Delta==2)
     figure
     subplot(222); 
     plot(real(s_hat),imag(s_hat),'x');grid; 
     title(['Equalizer output for \Delta=',num2str(Delta)]); 
 
 
     subplot(221); 
     plot(real(y),imag(y),'x');grid; 
     title('Received sequence y(i)'); 
   else
     subplot(224); 
     plot(real(s_hat),imag(s_hat),'x');grid; 
     title(['Equalizer output for \Delta=',num2str(Delta)]); 
 
 
     subplot(223); 
     plot(real(y),imag(y),'x');grid; 
     title('Received sequence y(i)'); 
   end
 
 
 
   % Estimate of m.m.s.e. 
 
    mmse(Delta+1)=0; 
  
    for j=1:N-Delta 
      mmse(Delta+1)=mmse(Delta+1)+(s_hat(j+Delta)-s(j))*(s_hat(j+Delta)-s(j))'; 
    end 
  
    mmse(Delta+1)=mmse(Delta+1)/(N-Delta); 
 
 
 
   % Number of errors 
 
 
     num_error(Delta+1)=0; 
 
     s_nl=zeros(1,N); 
     sq2=sqrt(2)/2;     
 
     for j=1:N-Delta 
 
       a=sign(real(s_hat(j+Delta))); 
       if a == 0   
         a=1; 
       end 
 
       b=sign(imag(s_hat(j+Delta))); 
       if b == 0   
         b=1; 
       end 
 
       s_nl(j)=sq2*(a+i*b);  % output of nonlinear device 
 
       if abs(s_nl(j)-s(j)) > 0.1 
         num_error(Delta+1) = num_error(Delta+1)+1; 
       end 
 
     end 
 
    Ko 
end 
 
 
figure 
subplot(221); 
stem(0:3,mmse); 
grid; 
title('MMSE'); 
xlabel('\Delta'); 
ylabel('mmse'); 
subplot(222); 
stem(0:3,num_error); 
grid; 
title('Number of errors'); 
xlabel('\Delta'); 
ylabel('errors'); 

num_error
 
 
 
 
