
% Solves parts C and D

% April 28, 1999
% Developed by Ali H. Sayed


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Remark: This code uses the following notation
% N+1  is number of feedforward taps  (text uses L)
% N    is order  of feedforward filter (text uses L-1)
% Q    is number of feedback taps (text uses Q)
% M+1  is number of channel taps (text uses M)
% M    is order of channel (text uses M+1)
%
% In other words, the text works with the number of taps and 
% denotes them by (L,Q,M). The program works with filter
% orders instead and denotes them by (N,Q,M).
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

close all
clear all

c=[0.5 1.2 1.5 -1];  % Channel coefficients: CAN BE CHANGED (EVEN ITS LENGTH)

var_s=1;          % variance of input data
var_v=0.078;      % variance of noise; CAN BE CHANGED
L=2000;           % number of symbols to be generated

M=length(c)-1;  % order of channel: number of taps - 1
N=12;           % order of feedforward filter: number of taps - 1
Q=2;            % number of feedback taps, which coincides with feedback order.

sq2=sqrt(2)/2;

s=sq2*(sign(randn(1,L))+i*sign(randn(1,L)));
                              % generates QPSK data

r=filter(c,1,s);              % Filters the data through the channel

y=r+sqrt(var_v/2)*randn(1,L)+sqrt(var_v/2)*i*randn(1,L); 
                              % adds complex noise to it



%%%%%%%%%%% Design of equalizer

col=zeros(N+1,1);
col(1)=c(1);

row=zeros(1,N+M+1);
row(1:M+1)=c;

CT = toeplitz(col,row); %  prewindowed Toeplitz channel matrix

R_v=var_v*eye(N+1,N+1);
R_under_s=var_s*eye(N+M+1,N+M+1);
R_s=var_s*eye(Q+1,Q+1);

R_y=CT*R_under_s*CT'+R_v;
IR_y=inv(R_y);


Delta_max= N+M;   % CAN BE CHANGED BUT SHOULD NOT EXCEED N+M

iter=0;

display('PLEASE WAIT FOR 16 ITERATIONS')


for Delta=0:Delta_max

  iter=iter+1

   % Calculate Rsy

   if Delta <= N+M-Q
     XT=CT';
     Rsy=var_s*XT(1+Delta:Delta+Q+1,1:N+1);
   else
     XT=CT';
     Rsy(1:N+M-Delta+1,:)=var_s*XT(1+Delta:N+M+1,1:N+1);
     Rsy(N+M-Delta+2:Q+1,:)=zeros(Delta-N-M+Q,N+1);
   end

   % Calculate R_delta and its inverse

    R_delta = R_s-Rsy*IR_y*Rsy';
    IR_delta = inv(R_delta);


   % Get the optimal coefficients for the DFE

    bopt=IR_delta(1,:)/IR_delta(1,1);  
    fopt=bopt*Rsy*IR_y;
    
    beq=bopt(2:Q+1);    

    mmse(Delta+1)=1/IR_delta(1,1);  % Theoretical m.m.s.e.


   % Calculate s_hat and s_check

    s_check=zeros(1,L+Q+1);
    s_hat=zeros(1,L+Q+1);
    y_1=zeros(1,L+Q+1);

    y_1=filter(fopt,1,y);   % Output of the feedforward filter


    % Let us compute the first Q outputs \hat{s} directly


     s_hat(1)=y_1(1);
     s_check(1)=dec(s_hat(1));  % Since initial conditions in B(z)
                                % are zero.

    %% We use .' in order to transpose the vector only
    %% without conjugation

     for j=2:Q+1
      s_hat(j)=y_1(j)-beq(1:j-1)*s_check(j-1:-1:1).';
      s_check(j)=dec(s_hat(j));
     end

     %% Now we compute the remaining equalizer outputs

     for j=Q+2:L
       s_hat(j)=y_1(j)-beq*s_check(j-1:-1:j-Q).';
       s_check(j)=dec(s_hat(j));
     end


    %% Number of errors    

     num_error(Delta+1)=0;
     sym_rate(Delta+1)=0;

     for j=1:L-Delta
       if abs(s_check(j+Delta)-s(j)) > 0.1
         num_error(Delta+1) = num_error(Delta+1)+1;
       end

     end

    sym_rate(Delta+1)=num_error(Delta+1)/(L-Delta);


   % Estimate of m.m.s.e.

    mmse_hat(Delta+1)=0;

    for j=1:L-Delta
     mmse_hat(Delta+1)=mmse_hat(Delta+1)+(s_hat(j+Delta)-s(j))*(s_hat(j+Delta)-s(j))';
    end
 
    mmse_hat(Delta+1)=mmse_hat(Delta+1)/(L-Delta);



    if Delta == 5   % Save data for Delta = 5

      input_device=s_hat;  %%% Save the input to the nonlinear
                           %%% device for Delay=5.
      after_dec=s_check;
      f_4 = fopt;
      b_4 = beq;

    end



end

Delta = 5;


%%%% We now plot all symbols in constellation 
%%%% form for both the received signal and 
%%%% the signal at the input of the decision
%%%% device

subplot(221);
plot(y,'.');grid;
title('Received sequence');

subplot(222);
plot(input_device,'.');grid;
title('Input of decision device');

num_error

figure
subplot(221)
stem(0:Delta_max,num_error);
grid;
title('Number of errors as a function of \Delta');
xlabel('\Delta');
ylabel('errors');

subplot(222)
plot(0:Delta_max,mmse,0:Delta_max,mmse_hat);  
title('Theoretical and estimated m.m.s.e. as a function of \Delta');
xlabel('\Delta');
ylabel('mmse');
legend('Theoretical m.m.s.e.','Estimated m.m.s.e.');
grid;


ff=conv(c,f_4);
Len=max(size(ff))-1;
figure
subplot(221)
stem(0:M,c);
title('channel')
grid;
subplot(222)
stem(0:Len,ff);
title('channel*feedforward')
grid;
axis tight
subplot(224)
stem(0:Len,[zeros(1,Delta) 0 -b_4 zeros(1,Len-Delta-2)]);
title('delayed feedback')
axis tight;
grid;

%ff
%-b_4
