
% Solves part G

% April 29, 1999
% Developed by Ali H. Sayed



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Remark: This code uses the following notation
% N+1  is number of feedforward taps  (text uses L)
% N    is order  of feedforward filter (text uses L-1)
% Q    is number of feedback taps (text uses Q)
% M+1  is number of channel taps (text uses M)
% M    is order of channel (text uses M+1)
%
% In other words, the text works with the number of taps and 
% denotes them by (L,Q,M). The program works with filter
% orders instead and denotes them by (N,Q,M).
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


close all
clear all

c=[0.5 1.2 1.5 -1];  % Channel coefficients: CAN BE CHANGED (EVEN ITS LENGTH)

var_s=1;          % variance of input data
L=20000;          % number of symbols to be generated

M=length(c)-1;  % length of channel
N=6;            % length of feedforward filter
Delta=4;        % Delay
Q=1;            % length of feedback filter

s=(sign(randn(1,L))+i*sign(randn(1,L)))/(sqrt(2));
                              % generates QPSK data

r=filter(c,1,s);              % Filters the data through the channel

CT=zeros(N+1,N+M+1);
R_y=zeros(N+1,N+1);
IR_y=zeros(N+1,N+1);
Rsy=zeros(Q+1,N+1);
R_delta=zeros(Q+1,Q+1);
IR_delta=zeros(Q+1,Q+1);

bopt=zeros(1,Q+1);
fopt=zeros(1,N+1);
beq=zeros(1,Q);


col=zeros(N+1,1);
col(1)=c(1);
row=zeros(1,N+M+1);
row(1:M+1)=c;

CT = toeplitz(col,row); %  prewindowed Toeplitz channel matrix

R_under_s=var_s*eye(N+M+1,N+M+1);
R_s=var_s*eye(Q+1,Q+1);



% Calculate Rsy
% THE VALUE OF DELTA CANNOT EXCEED N+M

XT=CT';
Rsy=var_s*XT(1+Delta:Delta+Q+1,1:N+1);



iter=0;

display('THIS TAKES SOME TIME...about 660 slow iterations')


for var_v=0.12:0.001:0.78

  y=r+sqrt(var_v/2)*randn(1,L)+sqrt(var_v/2)*i*randn(1,L); 
                              % adds complex noise to it


  iter=iter+1

  %%%% Design of equalizer 

  R_v=var_v*eye(N+1,N+1);
  R_y=CT*R_under_s*CT'+R_v;
  IR_y=inv(R_y);

  % Calculate R_delta and its inverse

  R_delta = R_s-Rsy*IR_y*Rsy';
  IR_delta = inv(R_delta);

 % Get the optimal coefficients for the DFE

   bopt=IR_delta(1,:)/IR_delta(1,1);  
   fopt=bopt*Rsy*IR_y;
    
   beq=bopt(2:Q+1);    

   mmse(iter)=1/IR_delta(1,1);  % Theoretical m.m.s.e.


   % Calculate s_hat and s_check

    s_check=zeros(1,L+Q+1);
    s_hat=zeros(1,L+Q+1);
    y_1=zeros(1,L+Q+1);

    y_1=filter(fopt,1,y);   % Output of the feedforward filter


    % Let us compute the first Q outputs \hat{s} directly


     s_hat(1)=y_1(1);
     s_check(1)=dec(s_hat(1));  % Since initial conditions in B(z)
                            % are zero.


     for j=2:Q+1
      s_hat(j)=y_1(j)-beq(1:j-1)*s_check(j-1:-1:1).';
      s_check(j)=dec(s_hat(j));
     end

     %% Now we compute the remaining equalizer outputs

     for j=Q+2:L
       s_hat(j)=y_1(j)-beq*s_check(j-1:-1:j-Q).';
       s_check(j)=dec(s_hat(j));
     end


    %% Number of errors    

     num_error(iter)=0;

 
     for j=1:L-Delta
       if abs(s_check(j+Delta)-s(j)) > 0.1
         num_error(iter) = num_error(iter)+1;
       end

     end




   % Estimate of m.m.s.e.

    mmse_hat(iter)=0;
 
    for j=1:L-Delta
     mmse_hat(iter)=mmse_hat(iter)+(s_hat(j+Delta)-s(j))*(s_hat(j+Delta)-s(j))';
    end
 
    mmse_hat(iter)=mmse_hat(iter)/(L-Delta);

end

%num_error

SNR=10*log10(norm(c)^2./(0.12:0.001:0.78));

figure
subplot(211)
plot(SNR,num_error/(L-Delta));
grid;
xlabel('SNR');
ylabel('SER');

%figure
%subplot(211)
%plot(0.05:0.05:2,mmse,0.05:0.05:2,mmse_hat); 
%title('Theoretical and estimated m.m.s.e. as a function of Q');
%xlabel('\sigma_v^2');
%ylabel('mmse');
%legend('Theoretical m.m.s.e.','Estimated m.m.s.e.');
%grid;
