
% Solves part (c). The value of gamma and the 
% variance of u, denoted by var_u, can be changed.
% Also, the values of the step-size, in the variable 
% mu, and the value of the initial weight estimate,
% in the variable w_init, can be changed.


% January 14, 2000
% Developed by Ali H. Sayed


clear all
close all

gamma=1;     % value of gamma
var_u=0.5;   % variance of u


w_min_1= sqrt(gamma/(3*var_u));   % One global minimum
w_min_2=-sqrt(gamma/(3*var_u));   % The other global minimum
w=1.5*w_min_2:0.001:1.5*w_min_1;     % Interval and cost
J = gamma^2 - (2*gamma*var_u)*w.^2 + (3*var_u*var_u)*w.^4;



%%% We now simulate different step-sizes and different initial 
%%% conditions.

N=20;  % 20 iterations (can be changed)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

mu  =  0.2;  % Step-size
w_init=0.3;  % Initial estimate

we = zeros(1,N);
Je = zeros(1,N);
we(1)=w_init;
Je(1)= gamma^2 - (2*gamma*var_u)*we(1)^2 + (3*var_u*var_u)*we(4)^4;

for i = 2:N,
 we(i) = we(i-1) + 4*mu*var_u*we(i-1)*(gamma - 3*var_u*we(i-1)*we(i-1));
 Je(i) = gamma^2 - (2*gamma*var_u)*we(i)^2 + (3*var_u*var_u)*we(i)^4;
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

mu  =   0.2;  % Step-size
w_init=-0.3;  % Initial estimate

we2 = zeros(1,N);
Je2 = zeros(1,N);
we2(1)=w_init;
Je2(1)= gamma^2 - (2*gamma*var_u)*we2(1)^2 + (3*var_u*var_u)*we2(4)^4;

for i = 2:N,
 we2(i) = we2(i-1) + 4*mu*var_u*we2(i-1)*(gamma - 3*var_u*we2(i-1)*we2(i-1));
 Je2(i) = gamma^2 - (2*gamma*var_u)*we2(i)^2 + (3*var_u*var_u)*we2(i)^4;
end;

figure
subplot(211)
ind=0:N-1;
plot(ind,we,'r',ind,we2,'-b');
title('Weight estimate trajectories with \mu=0.2 for two initial weight estimates');
xlabel('iteration');
ylabel('w_i');
legend('initial weight=  0.3  ','initial weight=-0.3   ');

subplot(223)
plot(w,J);
title('J(w_i) vs. w_i starting at 0.3');
xlabel('w_i');
ylabel('J(w_i)');
hold on;
plot(we(1:N),Je(1:N),'xr--')
zoom on
axis tight

subplot(224)
plot(w,J);
title('J(w_i) vs. w_i starting at -0.3');
xlabel('w_i');
ylabel('J(w_i)');
hold on;
plot(we2(1:N),Je2(1:N),'xr--')
zoom on
axis tight


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

mu  =  0.6;  % Step-size
w_init=0.3;  % Initial estimate

we = zeros(1,N);
Je = zeros(1,N);
we(1)=w_init;
Je(1)= gamma^2 - (2*gamma*var_u)*we(1)^2 + (3*var_u*var_u)*we(4)^4;

for i = 2:N,
 we(i) = we(i-1) + 4*mu*var_u*we(i-1)*(gamma - 3*var_u*we(i-1)*we(i-1));
 Je(i) = gamma^2 - (2*gamma*var_u)*we(i)^2 + (3*var_u*var_u)*we(i)^4;
end;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

mu  =   0.6;  % Step-size
w_init=-0.3;  % Initial estimate

we2 = zeros(1,N);
Je2 = zeros(1,N);
we2(1)=w_init;
Je2(1)= gamma^2 - (2*gamma*var_u)*we2(1)^2 + (3*var_u*var_u)*we2(4)^4;

for i = 2:N,
 we2(i) = we2(i-1) + 4*mu*var_u*we2(i-1)*(gamma - 3*var_u*we2(i-1)*we2(i-1));
 Je2(i) = gamma^2 - (2*gamma*var_u)*we2(i)^2 + (3*var_u*var_u)*we2(i)^4;
end;

figure
subplot(211)
plot(ind,we,'r',ind,we2,'-b');
title('Weight estimate trajectories with \mu=0.6 for two initial weight estimates');
xlabel('iteration');
ylabel('w_i');
legend('initial weight=  0.3   ','initial weight=-0.3   ');

subplot(223)
plot(w,J);
title('J(w_i) vs. w_i starting at 0.3');
xlabel('w_i');
ylabel('J(w_i)');
hold on;
plot(we(1:N),Je(1:N),'xr--')
zoom on
axis tight

subplot(224)
plot(w,J);
title('J(w_i) vs. w_i starting at -0.3');
xlabel('w_i');
ylabel('J(w_i)');
hold on;
plot(we2(1:N),Je2(1:N),'xr--')
zoom on
axis tight

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


mu  =   1;    % Step-size
w_init=-0.3;  % Initial estimate

we=zeros(1,5*N);
Je=zeros(1,5*N);

we(1)= w_init;
Je(1)= gamma^2 - (2*gamma*var_u)*we(1)^2 + (3*var_u*var_u)*we(4)^4;

for i = 2:5*N,
 we(i) = we(i-1) + 4*mu*var_u*we(i-1)*(gamma - 3*var_u*we(i-1)*we(i-1));
 Je(i) = gamma^2 - (2*gamma*var_u)*we(i)^2 + (3*var_u*var_u)*we(i)^4;
end;

 
figure
subplot(211)
plot(0:5*N-1,we);
title('Weight estimate trajectory with \mu=1 and initial weight estimate at -0.2');
xlabel('iteration');
ylabel('w_i');
axis tight

subplot(212)
plot(w,J);
title('Evolution of weight estimate x cost for \mu=1 and initial weight estimate at -0.2');
xlabel('w_i');
ylabel('J(w_i)');
hold on;
plot(we(1:N),Je(1:N),'xr--')
zoom on
axis tight

