%Adaptive equalization project
%Solves part E
%Developed by A. H. Sayed

clear all;  
close all

qam = 64 % Choose from 4/16/64/256
SNR = 30 % SNR at output of channel
Nf=10;   %feedforward taps
Nb=2;   %feedback taps
Delta =7; %approx. delay by channel + half length of feedforward equalizer
mu = 0.4;
epsilon = 1e-6; 

training = 500;  % length of training with BPSK data
decision_directed = 5000; % length of decision-directed with QAM data
N=training + decision_directed; % total number of symbols
train_data = zeros(1,training+Delta); %training data
s=zeros(1,N);


%Channel
 channel=[0.5 1.2 1.5 -1]; % from chapter 3. Use 35taps/500training for 256, 30dBSNR, Delta=10

%Noise variance during decision-directed
%QAM symbols don't have unit-variance
switch qam
case 4
   sigma_s = sqrt(2);
case 16
   sigma_s = sqrt(10);
case 64
   sigma_s = sqrt(42);
case 256
   sigma_s = sqrt(170);
end
sigma_v_dd = sqrt((sigma_s)^2*norm(channel)^2/10^(SNR/10)); % during decision-directed
sigma_v_tt = sqrt(norm(channel)^2/10^(SNR/10)); % during training

v=zeros(1,N);
v(1:training)=(sigma_v_tt/sqrt(2))*(randn(1,training)+j*randn(1,training));
v(training:N)=(sigma_v_dd/sqrt(2))*(randn(1,N-training+1)+j*randn(1,N-training+1));


%Data
s(1:training)=(sign(randn(1,training))+j*sign(randn(1,training)))/(sqrt(2)); 
train_data(Delta+1:training+Delta)=s(1:training); % QPSK training data
   % generates QPSK data; it has unit variance
   % The training vector starts with Delta zeros.
   
for i=1:decision_directed
 x=rand;   
 xint=round((qam-1)*x); % generates integers, e.g., between 0 and 255
 xcoor=modmap(xint,1,1,'qam',qam); % gives real and imaginary coordinates
 s(training+i)=xcoor(1) + j*xcoor(2); % combines them into a complex number
end 
 
y=filter(channel,1,s); % filters data through channel
r=y+v; 

% Equalizer initialization
wf = zeros(Nf,1);
wb = zeros(Nb,1);
w=[wf;wb];
uf = zeros(1,Nf);
ub = zeros(1,Nb);
e = zeros(1,N); % error vector
num_errors=0;

%  Adaptive Equalization

% Training mode
for i = 1:training+Delta
   uf  = [r(i) uf(1:Nf-1)];
   hat_s(i) = uf*wf + ub*wb;
   d(i) = train_data(i); % training
   e(i) = d(i)- hat_s(i);
   u = [uf ub];   
   w = w + mu*e(i)*u'/(norm(u)^2+epsilon);
   wf = w(1:Nf);
   wb = w(Nf+1:Nf+Nb);
end


% Decision-directed mode
for i = training+Delta+1:N
   uf  = [r(i) uf(1:Nf-1)];
   hat_s(i) = uf*wf + ub*wb;
   switch qam 
    case 4
      check_s(i)=slicer4(hat_s(i));
    case 16
      check_s(i)=slicer16(hat_s(i));
    case 64
      check_s(i)=slicer64(hat_s(i));
    case 256
      check_s(i)=slicer256(hat_s(i));
   end
   
   d(i) = check_s(i); % decision_directed
   e(i) = d(i)- hat_s(i);
   u = [uf ub];
   w = w + mu*e(i)*u'/(norm(u)^2+epsilon);
   wf = w(1:Nf);
   wb = w(Nf+1:Nf+Nb);
   
   if (check_s(i) ~= s(i-Delta))
     num_errors = num_errors + 1;
   end
   
end  


for k = 1:N
  error_curve(k) = e(k)*e(k)';   % squared error curve
end

  
%%%% We plot the scatter diagrams of 
%%%% both the transmitted and received signals 
 
figure
subplot(221); 
plot(real(train_data(Delta+1:training)),imag(train_data(Delta+1:training)),'.');
grid; 
title('Training sequence'); 

subplot(222); 
plot(real(s(training+Delta+1:N)),imag(s(training+Delta+1:N)),'.');
grid; 
title('Transmitted sequence'); 
 

figure
subplot(221); 
plot(real(r(training+Delta+1:N)),imag(r(training+Delta+1:N)),'.');grid; 
title('Received sequence'); 
 

subplot(222); 
plot(real(hat_s(training+Delta+1:N)),imag(hat_s(training+Delta+1:N)),'.');grid; 
title('Equalizer output');


 
num_errors
 
 
 
 

