%Adaptive equalization project
%Solves part G
%Developed by A. H. Sayed

clear all;  
close all

qam = 4 % Choose from 4/16/64/256
SNR = 40 % SNR at output of channel
equalizer_length = 55;
Delta =30; %approx. delay by channel + half length of equalizer
mu = 0.4;
epsilon = 1e-6; 
delta=1000;
lambda=0.995;

load channel % dispersive channel
M=length(channel)-1;
figure
subplot(221),
plot(0:M,channel); grid;
title('Impulse response');
subplot(222); 
[H,W]=freqz(channel,1,512);
plot(W,20*log10(abs(H)));
title('Frequency response'); grid;
xlabel('\omega');
ylabel('dB')
axis tight;


       
for alg=1:2
  % alg=1 --> RLS
  % alg=2 --> NLMS
 if alg==1
     disp('RLS')
     training = 100;  % length of training with BPSK data
     decision_directed = 5000; % length of decision-directed with QAM data
     N=training + decision_directed; % total number of symbols
     train_data = zeros(1,training+Delta); %training data
     s=zeros(1,N);
 else
     disp('NLMS')
     training = 2000;  % length of training with BPSK data
     decision_directed = 5000; % length of decision-directed with QAM data
     N=training + decision_directed; % total number of symbols
     train_data = zeros(1,training+Delta); %training data
     s=zeros(1,N);
     training=2000;
 end


%Noise variance during decision-directed
%QAM symbols don't have unit-variance
switch qam
case 4
   sigma_s = sqrt(2);
case 16
   sigma_s = sqrt(10);
case 64
   sigma_s = sqrt(42);
case 256
   sigma_s = sqrt(170);
end
sigma_v_dd = sqrt((sigma_s)^2*norm(channel)^2/10^(SNR/10)); % during decision-directed
sigma_v_tt = sqrt(norm(channel)^2/10^(SNR/10)); % during training

v=zeros(1,N);
v(1:training)=(sigma_v_tt/2)*(randn(1,training)+j*randn(1,training));
v(training:N)=(sigma_v_dd/2)*(randn(1,N-training+1)+j*randn(1,N-training+1));


%Data
s(1:training)=(sign(randn(1,training))+j*sign(randn(1,training)))/(sqrt(2)); 
train_data(Delta+1:training+Delta)=s(1:training); % QPSK training data
   % generates QPSK data; it has unit variance
   % The training vector starts with Delta zeros.
   
for i=1:decision_directed
 x=rand;   
 xint=round((qam-1)*x); % generates integers, e.g., between 0 and 255
 xcoor=modmap(xint,1,1,'qam',qam); % gives real and imaginary coordinates
 s(training+i)=xcoor(1) + j*xcoor(2); % combines them into a complex number
end 
 
y=filter(channel,1,s); % filters data through channel
r=y+v; 

% Equalizer initialization
w  = zeros(equalizer_length,1); % equalizer coefficients (column) 
u  = zeros(1,equalizer_length); % regressor vector (row) 
e = zeros(1,N); % error vector
num_errors=0;

P=delta*eye(equalizer_length,equalizer_length);
lambda_inv=1/lambda;

%  Adaptive Equalization

% Training mode
for i = 1:training+Delta
   u  = [r(i) u(1:equalizer_length-1)];
   hat_s(i) = u*w;
   d(i) = train_data(i); % training
   e(i) = d(i)- hat_s(i);
   if alg == 2
    w = w + mu*e(i)*u'/(norm(u)^2+epsilon);
   else
    gamma = 1 + lambda_inv*u*P*u';
    P = lambda_inv*(P-(lambda_inv*P*u'*u*P/gamma));
    w=w+P*u'*e(i);
   end
end


% Decision-directed mode
for i = training+Delta+1:N
   u  = [r(i) u(1:equalizer_length-1)];
   hat_s(i) = u*w;
   switch qam 
    case 4
      check_s(i)=slicer4(hat_s(i));
    case 16
      check_s(i)=slicer16(hat_s(i));
    case 64
      check_s(i)=slicer64(hat_s(i));
    case 256
      check_s(i)=slicer256(hat_s(i));
   end
   
   d(i) = check_s(i); % decision_directed
   e(i) = d(i)- hat_s(i);
   if alg == 2
    w = w + mu*e(i)*u'/(norm(u)^2+epsilon);
   else
    gamma = 1 + lambda_inv*u*P*u';
    P = lambda_inv*(P-(lambda_inv*P*u'*u*P/gamma));
    w=w+P*u'*e(i);
   end
   
   if (check_s(i) ~= s(i-Delta))
     num_errors = num_errors + 1;
   end
   
end  

if alg == 1
  figure
   subplot(331); 
   plot(real(s(Delta+1:training)),imag(s(Delta+1:training)),'.');
   grid; 
   title('Training sequence'); 

   subplot(332); 
   plot(real(hat_s(training+Delta+1:N)),imag(hat_s(training+Delta+1:N)),'.');grid; 
   title('RLS');
   
   w_rls=w;
else
   subplot(333); 
   plot(real(hat_s(training+Delta+1:N)),imag(hat_s(training+Delta+1:N)),'.');grid; 
   title('NLMS');
end  
  
num_errors



end 


   figure
   subplot(331); 
   [H,W]=freqz(channel,1,512);
   plot(W,20*log10(abs(H)));
   title('channel'); grid;
   xlabel('\omega');
   ylabel('dB')
   axis tight;
   subplot(332); 
   [H,W]=freqz(w_rls,1,512);
   plot(W,20*log10(abs(H)));
   title('equalizer'); grid;
   xlabel('\omega');
   ylabel('dB')
   axis tight;

   z=conv(channel,w_rls);
   subplot(333); 
   [H,W]=freqz(z,1,512);
   plot(W,20*log10(abs(H)));
   title('combination'); grid;
   xlabel('\omega');
   ylabel('dB')
   axis tight;
   

   
   subplot(334); 
   plot(channel);
   xlabel('i');
   grid
   axis tight;
   subplot(335); 
   plot(real(w_rls));
   xlabel('i');
   grid;
   axis tight;

   subplot(336); 
   plot(real(z));
   xlabel('i');
   axis tight;
   grid
 
 

