%Blind adaptive equalization project
%Solves part A
%Developed by A. H. Sayed

clear all;  
close all

SNR = 30 % SNR at output of channel
equalizer_length = 35;
mu = 0.001;
N=10000; % total number of symbols
s=zeros(1,N);
gamma=1;

%Channel
 channel=[0.5 1.2 1.5 -1]; % from chapter 3. 

%Noise
sigma_s=1;
sigma_v= sqrt((sigma_s)^2*norm(channel)^2/10^(SNR/10)); 
v=zeros(1,N);
v(1:N)=(sigma_v/2)*(randn(1,N)+j*randn(1,N));


%Data QPSK
s(1:N)=(sign(randn(1,N))+j*sign(randn(1,N)))/(sqrt(2)); 
y=filter(channel,1,s); % filters data through channel
r=y+v; % received data

% Equalizer initialization
w  = zeros(equalizer_length,1); % equalizer coefficients (column) 
w(ceil(equalizer_length/2))= 1;  % initial condition cannot be zero for CMA2-2!
u  = zeros(1,equalizer_length); % regressor vector (row) 
e = zeros(1,N); % error vector
num_errors=0;

% Blind adaptive equalization
for i = 1:N
   u  = [r(i) u(1:equalizer_length-1)];
   hat_s(i) = u*w;
   check_s(i) = dec(hat_s(i));
   e(i) = hat_s(i)*(gamma-(hat_s(i)*hat_s(i)')); % CMA2-2
   w = w + mu*e(i)*u';
end
  
%%%% We plot the scatter diagrams of 
%%%% both the transmitted and received signals 
 
gap = 2000;  % Only count errors after this initial number of iterations to
             % allow for equalizer convergence.
figure
subplot(331); 
plot(real(s(1+gap:N)),imag(s(1+gap:N)),'.');
grid; 
title('Transmitted sequence'); 
 

subplot(332); 
plot(real(r(1+gap:N)),imag(r(1+gap:N)),'.');grid; 
title('Received sequence'); 
 

subplot(333); 
plot(real(hat_s(1+gap:N)),imag(hat_s(1+gap:N)),'.');grid; 
title('Equalizer output');

z=conv(channel,w);
m1=max(size(channel));
m2=max(size(w));
m3=max(size(z));
figure
subplot(331)
plot(0:m1-1,channel)
title('channel')
xlabel('i')
axis tight
subplot(332)
plot(0:m2-1,real(w))
title('equalizer')
xlabel('i');
axis tight
subplot(333)
plot(0:m3-1,real(z))
title('combination')
xlabel('i');
axis tight

[val d]=max(real(z)); 
delay=d-1; % since entries of z are numbered starting from 1.
for i=gap+1:N
  if abs(check_s(i) - s(i-delay))>0.1
   num_errors = num_errors + 1;
  end
end
num_errors
 
 
figure
subplot(411)
stairs(real(s(gap+1-delay:gap+1+50-delay)));
legend('real(s)')
subplot(412)
stairs(real(check_s(gap+1:gap+1+50)));
legend('real(decision)')
subplot(413)
stairs(imag(s(gap+1-delay:gap+1+50-delay)));
legend('imag(s)')
subplot(414)
stairs(imag(check_s(gap+1:gap+1+50)));
legend('imag(decision)')

 
 

