% Solves part C, D, and E
% A. H. Sayed
% March 2002.

close all
clear all

var_n = 1e-3;
sqn = sqrt(var_n);

Ts = 0.8e-6; % sampling frequency is 1.25MHz
Ns = 30000;
fd = 10; % Doppler frequency

ray1 = rayleigh(Ts,Ns,fd);
ray2 = rayleigh(Ts,Ns,fd);
N = Ns;

learn = zeros(1,N);
mu = 0.01;
L = 100;
M = 5;
R = eye(M,M);

for k = 1:1
 k   
 input = randn(1,N);
 v = sqn*randn(1,N); 
 e = zeros(1,N);
 u = zeros(1,M);
 w = zeros(M,1);
   
   for j=1:N
     wo = [0 0 ray1(j) 0 ray2(j)]';
     u = [input(j) u(1:M-1)];
     d = u*wo + v(j);
     e(j) = d - u*w;
     w = w + mu*u'*e(j);
     if (k == 1)
       amp(j) = abs(w(3)); % saves one trajectory
     end
   end 
   
   if (k==1)
     figure
     subplot(221)
     plot(1:N,abs(ray1),'r',1:N,amp,'b-');
     xlabel('i');
     ylabel('Amplitude')
     axis tight
     grid
     subplot(222)
     plot(1000:1100,abs(ray1(1000:1100)),'r',1000:1100,amp(1000:1100),'b-');
     xlabel('i');
     ylabel('Amplitude')
     axis tight
     grid

   end
   
   learn = learn + abs(e).^2;
end
learn = learn/L;
mse = mean(learn(N-100:N))


r1 = bessel(0,2*pi*fd*Ts); 
q = 1-r1*r1;
Q = q*eye(M,M);
thy = var_n + (mu*var_n*trace(R) + (1/mu)*trace(Q))/(2-mu*trace(R));

X_alpha = (eye(M,M)-mu*R)*inv(r1 - inv(eye(M,M)-mu*R));
b = trace((eye(M,M)+2*(1-r1)*X_alpha)*Q);
thy2 = var_n + (mu*var_n*trace(R) + (1/mu)*b)/(2-mu*trace(R));
[mse thy thy2]

figure
plot(10*log10(learn(1:1000)));
xlabel('i')
ylabel('MSE (dB)')