% Solves part F
% A. H. Sayed
% March 2002.

close all
clear all

var_n = 1e-3;
sqn = sqrt(var_n);

Ts = 0.8e-6; % sampling frequency is 1.25MHz
Ns = 60000;
fd = [10 15 20 25]; % Doppler frequencies
K = length(fd);

M = 5; % number of taps
R = eye(M,M);

for i=1:K % iterate over doppler frequencies
  ray1 = rayleigh(Ts,Ns,fd(i));
  ray2 = rayleigh(Ts,Ns,fd(i));
  N = Ns;

  learn = zeros(1,N);
  mu = 0.01;
  L = 50;
  M = 5;
  R = eye(M,M);

  for k = 1:L
   k   
   input = randn(1,N);
   v = sqn*randn(1,N); 
   e = zeros(1,N);
   u = zeros(1,M);
   w = zeros(M,1);

   for j=1:N
     wo = [0 0 ray1(j) 0 ray2(j)]';
     u = [input(j) u(1:M-1)];
     d = u*wo + v(j);
     e(j) = d - u*w;
     w = w + mu*u'*e(j);
   end 
   learn = learn + abs(e).^2;
  end
  learn = learn/L;
  mse(i) = mean(learn(N-100:N))
 end

 for i=1:K
  f(i) = bessel(0,2*pi*fd(i)*Ts); % coefficients 
  q(i) = (1-(f(i)*f(i))); % variances
  Q = q(i)*eye(M,M);
  thy(i) = var_n + (mu*var_n*trace(R) + (1/mu)*trace(Q))/(2);

  X_alpha = (eye(M,M)-mu*R)*inv(f(i) - inv(eye(M,M)-mu*R));
  b = trace((eye(M,M)+2*(1-f(i))*X_alpha)*Q);
  thy2(i) = var_n + (mu*var_n*trace(R) + (1/mu)*b)/(2-mu*trace(R));
 end

[mse' thy' thy2']

 figure
 plot(fd,10*log10(mse),'r-o',fd,10*log10(thy),'g-d',fd,10*log10(thy2),'b-v')
 ylabel('MSE (dB)')
 xlabel('f_D (Hz)');
 legend('Simulation','Theory','Theory with \alpha');
 axis tight
 grid
 axis([10 25 -30 -29])