
% Adaptive channel estimation
% Solves part B

% Written by Ali H. Sayed 
% September 2002


close all
clear all       % clears all variables


M = 64;
load channel;
N = 1000;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% Standard FTF (quantized)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

L = 100; % 100 experiments
learn_ftf = zeros(N,1);
SNR = 30;
var_v = 10^(-SNR/10);

Br = 36;    % number of data bits (including sign bit)
Bc = 36;    % number of coefficient bits (including sign bit)

for ll=1:L
 ll    

 input = randn(1,N);
 noise = sqrt(var_v)*randn(1,N);

 w = zeros(M,1);   
 u = zeros(1,M);
 e = zeros(N,1);

 wf = zeros(M,1); % forward prediction vector wf
 wb = zeros(M,1); % backward prediction vector wb
 w  = zeros(M,1); % channel taps
 g  = zeros(M,1); % gain vector
 reg = zeros(1,M); % regressor

 lambda = 1- 2^(-10); % forgetting factor -- usually in the range (1-1/3M,1-1/10M)
 inv_delta = 2^(-11); % inverse of regularization factor
 gamma   = 1; % conversion factor
 xif = inv_delta/lambda; % forward prediction energy
 xib = inv_delta/lambda^(M+1); % backward prediction energy

 
 for i=1:N 
    fa = input(i) - quantize(u*wf,Br);  % a-priori forward error
    fa = quantize(fa,Br);

    f  = gamma*fa;             % a-posteriori forward error
    f = quantize(f,Br);

    xifo = quantize(xif,Br);   % save forward energy

    xif = lambda*xif + f*fa';    % time-update of forward energy
    xif = quantize(xif,Br);

    gamma1 = gamma*lambda*xifo/xif; % order- and time-update of conversion factor
    gamma1 = quantize(gamma1,Br);

    g1 = [0;g] + quantize_v(fa'/(lambda*xifo)*[1;-wf],Bc); % order- and time-update of gain vector
    g1 = quantize_v(g1,Bc);

    v = g1(M+1);  % last entry of gain vector
    wf = wf + quantize_v(f*g,Bc); % time-update of forward prediction taps
    wf = quantize_v(wf,Bc);

    ba = lambda*xib*v;  % a-priori backward error
    ba = quantize(ba,Br);

    if (1-ba*gamma1*v' <= 0)
      ind = num2str(i); 
      str = ['rescue needed but not used at iteration ',ind]; 
      disp(str) % message
    end

    
    gamma = gamma1/(1-ba*gamma1*v'); % order downdate of conversion factor
    gamma = quantize(gamma,Br);

    g = g1(1:M) + quantize_v(v*wb,Bc);          % order-downdate of gain vector 
    g = quantize_v(g,Bc);

    b = gamma*ba;                % a-posteriori backward error
    b = quantize(b,Br);

    xib = lambda*xib + b*ba';      % time-update of backward energy
    xib = quantize(xib,Br);

    wb = wb + quantize_v(b*g,Bc);                % time-update of backward prediction taps
    wb = quantize_v(wb,Bc);

    u = [input(i) u(1:M-1)];
    reference(i) = u*channel + noise(i);
    hat_d = quantize(u*w,Br);

    e(i) = reference(i) - hat_d; 
    e(i) = quantize(e(i),Br);
    ep = e(i)*gamma;   % a-posteriori estimation error
    ep = quantize(ep,Br);

    w = w + quantize_v(ep*g,Bc);      % time-update of weight vector
    w = quantize_v(w,Bc);
    
    
 end  
 learn_ftf = learn_ftf + abs(e).^2;
end

learn_ftf = 10*log10(learn_ftf/L);

save learn_ftf learn_ftf

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% FTF with rescue mechanism (quantized)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

L = 100; % 100 experiments
learn_ftf_r = zeros(N,1);
SNR = 30;
var_v = 10^(-SNR/10);

Br = 36;    % number of data bits (including sign bit)
Bc = 36;    % number of coefficient bits (including sign bit)

for ll=1:L
 ll    

 input = randn(1,N);
 noise = sqrt(var_v)*randn(1,N);

 w = zeros(M,1);   
 u = zeros(1,M);
 e = zeros(N,1);

 wf = zeros(M,1); % forward prediction vector wf
 wb = zeros(M,1); % backward prediction vector wb
 w  = zeros(M,1); % channel taps
 g  = zeros(M,1); % gain vector
 reg = zeros(1,M); % regressor

 lambda = 1- 2^(-10); % forgetting factor -- usually in the range (1-1/3M,1-1/10M)
 inv_delta = 2^(-11); % inverse of regularization factor
 gamma   = 1; % conversion factor
 xif = inv_delta/lambda; % forward prediction energy
 xib = inv_delta/lambda^(M+1); % backward prediction energy

 
 for i=1:N 
    fa = input(i) - quantize(u*wf,Br);  % a-priori forward error
    fa = quantize(fa,Br);

    f  = gamma*fa;             % a-posteriori forward error
    f = quantize(f,Br);

    xifo = quantize(xif,Br);   % save forward energy

    xif = lambda*xif + f*fa';    % time-update of forward energy
    xif = quantize(xif,Br);

    gamma1 = gamma*lambda*xifo/xif; % order- and time-update of conversion factor
    gamma1 = quantize(gamma1,Br);

    g1 = [0;g] + quantize_v(fa'/(lambda*xifo)*[1;-wf],Bc); % order- and time-update of gain vector
    g1 = quantize_v(g1,Bc);

    v = g1(M+1);  % last entry of gain vector
    wf = wf + quantize_v(f*g,Bc); % time-update of forward prediction taps
    wf = quantize_v(wf,Bc);

    ba = lambda*xib*v;  % a-priori backward error
    ba = quantize(ba,Br);

    
    if (1-ba*gamma1*v' > 0)
      gamma = gamma1/(1-ba*gamma1*v'); % order downdate of conversion factor
      gamma = quantize(gamma,Br);
      
      g = g1(1:M) + quantize_v(v*wb,Bc);   % order-downdate of gain vector 
      g = quantize_v(g,Bc);
      
      b = gamma*ba;                % a-posteriori backward error
      b = quantize(b,Br);
      
      xib = lambda*xib + b*ba';      % time-update of backward energy
      xib = quantize(xib,Br);
      
      wb = wb + quantize_v(b*g,Bc);  % time-update of backward prediction taps
      wb = quantize_v(wb,Bc);
      
     else % rescue procedure
       ind = num2str(i); 
       str = ['rescue procedure used at iteration ',ind]; 
       disp(str) % message
       gamma = 1;
       g  = zeros(M,1);
       wb = g;
       wf = g;
       b  = ba;
       xif = inv_delta/lambda;
       xib = inv_delta/lambda^(M+1);
     end

    u = [input(i) u(1:M-1)];
    reference(i) = u*channel + noise(i);
    hat_d = quantize(u*w,Br);

    e(i) = reference(i) - hat_d; 
    e(i) = quantize(e(i),Br);
    ep = e(i)*gamma;   % a-posteriori estimation error
    ep = quantize(ep,Br);

    w = w + quantize_v(ep*g,Bc);      % time-update of weight vector
    w = quantize_v(w,Bc);
    
    
 end  
 learn_ftf_r = learn_ftf_r + abs(e).^2;
end

learn_ftf_r = 10*log10(learn_ftf_r/L);

save learn_ftf_r learn_ftf_r

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% Stabilized FTF  (quantized)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

L = 100; % 100 experiments
learn_sftf = zeros(N,1);
SNR = 30;
var_v = 10^(-SNR/10);

Br = 36;    % number of data bits (including sign bit)
Bc = 36;    % number of coefficient bits (including sign bit)

for ll=1:L
 ll    

 input = randn(1,N);
 noise = sqrt(var_v)*randn(1,N);

 w = zeros(M,1);   
 u = zeros(1,M);
 e = zeros(N,1);

 wf = zeros(M,1); % forward prediction vector wf
 wb = zeros(M,1); % backward prediction vector wb
 w  = zeros(M,1); % channel taps
 g  = zeros(M,1); % gain vector
 reg = zeros(1,M); % regressor

 lambda = 1- 2^(-10); % forgetting factor -- usually in the range (1-1/3M,1-1/10M)
 inv_delta = 2^(-11); % inverse of regularization factor
 gaminv   = 1; % conversion factor
 xifinv = lambda/inv_delta; % forward prediction energy
 xib = inv_delta/lambda^(M+1); % backward prediction energy

 k = [1.5 2.5 1]; % gains for convex combinations 

 for i=1:N 
    
   fa = input(i) - quantize(u*wf,Br);  % a-priori forward error
   fa = quantize(fa,Br);

   f  = fa/gaminv;  % a-posteriori forward error 
   f = quantize(f,Br);

   g1 = [0;g] + quantize_v(fa'*xifinv/lambda*[1;-wf],Bc); % order- and time-update of gain vector
   g1 = quantize_v(g1,Bc);
   
   v1 = g1(M+1); 
   v2 = g1(1); 
   gaminva = gaminv + v2*fa'; 
   gaminva = quantize(gaminva,Br);
   
   xifinv  = xifinv/lambda - abs(v2)^2/gaminva; 
   xifinv = quantize(xifinv,Br);
   
   wf = wf + quantize_v(f*g,Bc); 
   wf = quantize_v(wf,Bc);
   
   ba1 = lambda*xib*v1;
   ba1 = quantize(ba1,Br);
   
   ba2 = u(M) - quantize([input(i) u(1:M-1)]*wb,Br); 
   ba2 = quantize(ba2,Br);
   
   for ii=1:3 
    ba3(ii) = ba2*k(ii) + ba1*(1-k(ii)); 
    ba3(ii) = quantize(ba3(ii),Br);
   end 
   gaminv1 = gaminva - v1*ba3(3)'; 
   gaminv1 = quantize(gaminv1,Br);
 
   if gaminv1 > 0 
     for ii=1:2 
       b(ii) = ba3(ii)/gaminv1; 
       b(ii) = quantize(b(ii),Br);
     end 
     xib = lambda*xib + b(2)*ba3(2)';         
     xib = quantize(xib,Br);
     
     g = g1(1:M) + quantize_v(v1*wb,Bc); 
     g = quantize_v(g,Bc);     
     
     wb = wb + quantize_v(b(1)*g,Bc); 
     wb = quantize_v(wb,Bc);

     gaminv = 1/(lambda^(M)*xifinv*xib); 
     gaminv = quantize(gaminv,Br);
     
   else % rescue procedure 
     ind = num2str(i); 
     str = ['rescue procedure used at iteration ',ind]; 
     disp(str) % message
       
     gaminv = 1;
     g  = zeros(M,1);
     wb = g;
     wf = g;
     xif = lambda/inv_delta; 
     xib = inv_delta/lambda^(M+1);
   end 
 
  u = [input(i) u(1:M-1)]; 
  reference(i) = u*channel + noise(i);
  hat_d = quantize(u*w,Br); 
  
  e(i) = quantize(reference(i) - hat_d,Br); 
  
  ep = e(i)/gaminv; % a-posteriori estimation error
  ep = quantize(ep,Br);

  w = w + quantize_v(ep*g,Bc);      % time-update of weight vector
  w = quantize_v(w,Bc);
 end    
 learn_sftf = learn_sftf + abs(e).^2;
end

learn_sftf = 10*log10(learn_sftf/L);

save learn_sftf learn_sftf
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Fast array (quantized)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Le = 100; % 100 experiments
learn_array = zeros(N,1);
SNR = 30;
var_v = 10^(-SNR/10);

for ll=1:Le
 ll    

 input = randn(1,N);
 noise = sqrt(var_v)*randn(1,N);

 w = zeros(M,1);   
 u = zeros(1,M);
 e = zeros(N,1);

 lambda = 1- 2^(-10);
 eta = 2^(11); 
 
 L = zeros(M+1,2); 
 L(1,1) = sqrt(eta*lambda);
 L(M+1,2)=sqrt(eta*lambda)*lambda^(M/2);
 
 g  = zeros(M,1);  % gain vector
 gammainvsq   = 1; % square-root of inverse conversion factor
 k = g*gammainvsq; % normalized gain

 A = zeros(M+2,3); % pre-array
 B = zeros(M+2,3); % post-array
 
 ux = zeros(1,M+1); % extended regressor

 Br = 36;    % number of data bits (including sign bit)
 Bc = 36;    % number of coefficient bits (including sign bit)

 
 for i=1:N
   ux = [input(i) u] ;
   A = [gammainvsq quantize_v(ux*L,Br); [0;k] L];

   [c,s] = givens(A(1,1),A(1,2));  % determine Givens rotation c and s parameters
   c = quantize(c,Br);
   s = quantize(s,Br);
   
   B = A*[c s 0;conj(s) -c 0;0 0 1]; % apply Givens rotation
   B = quantize_v(B,Br);
   
   [c,s] = hyperbolic(B(1,1),B(1,3)); % determine hyperbolic rotation parameters
  %[c,s] = hyperbolic(B(M+2,1),B(M+2,3)); % determine hyperbolic rotation parameters  (second version)

   c = quantize(c,Br);
   s = quantize(s,Br);
   
   B = B*[c 0 -s;0 1 0;-conj(s) 0 c]; % apply hyperbolic rotation
   B = quantize(B,Br);
   
   L = B(2:M+2,2:3)/sqrt(lambda);
   L = quantize_v(L,Br);
   
   gammainvsq = B(1,1);
   k = B(2:M+1,1);  
   
   u = [input(i) u(1:M-1)];
   reference(i) = u*channel + noise(i);
   e(i) = reference(i)- quantize(u*w,Br);
   e(i) = quantize(e(i),Br);
   w = w + quantize_v((k/gammainvsq)*e(i),Bc);    
   w = quantize_v(w,Bc);
 end

 learn_array = learn_array + abs(e).^2;
end

learn_array = 10*log10(learn_array/Le);

save learn_array learn_array

figure
subplot(221)
plot(1:N,learn_ftf,'b-');
ylabel('E|e(i)|^2 in dB');
title('FTF');
axis tight
grid

subplot(222)
plot(1:N,learn_ftf_r,'b-');
title('FTF with rescue');
axis tight
grid

subplot(223)
plot(1:N,learn_sftf,'b-');
ylabel('E|e(i)|^2 in dB');
xlabel('i');
title('Stabilized FTF');
axis tight
grid

subplot(224)
plot(1:N,learn_array,'b-');
title('Fast array');
xlabel('i');
axis tight
grid

