
% Solves parts A, B, and C
% Transient performance of LMS with Gaussian regressors
% A. H. Sayed
% March 2002

 close all;
 clear all;

 N = 10000;  % number of iterations
 
 M = 5; % number of taps
 wo = randn(M,1);
 wo = wo/norm(wo);

 var_n = 0.001; % Gaussian noise
 sqn   = sqrt(var_n);

 mu = 0.01;

 rho = 5;  % desired eigenvalue spread
 lambda_min = 1;   % generate positive eigenvalues in the interval
 lambda_max = rho; % [lambda_min; lambda_max];
 eigenv = lambda_min + (lambda_max-lambda_min)*rand(M-2,1);
 R = diag([lambda_min;eigenv;lambda_max]); % Covariance matrix; it is diagonal = Lambda
 sqR = sqrt(R);

 Lambda = R; % Since R is diagonal; therefore U = I in this problem
 lambda = [lambda_min;eigenv;lambda_max]; % a column of eigenvalues
 

 
 %%%%%%%%%%%%%%%%% GENERATE THEORETICAL MSD CURVE
 
 F_bar = (eye(M,M) - 2*mu*Lambda + mu^2*Lambda^2) + mu^2*lambda*lambda';
 wo_bar = wo;
 
 msd(1) = norm(wo)^2; % time indexing starts at 1 in matlab!
 Fa = eye(M,M);
 for i=1:N-1
   W1 = diag(Fa*(eye(M,M)-F_bar)*ones(M,1)); % undo the vec operation to generate the weight matrix
   b = mu*mu*var_n*(lambda'*Fa)*ones(M,1);
   c = wo_bar'*W1*wo_bar;    
   msd(i+1) = msd(i) + b - c; % theoretical MSD curve for LMS
   Fa =  F_bar*Fa;
end



%%%%%%%%%%%%%%%%% GENERATE ENSEMBLE-AVERAGE  MSD CURVE

L = 30;

w  = zeros(M,1); % initial weight estimate
u  = zeros(1,M); % initial regressor

      
msd_curve_sim = zeros(1,N); % simulated MSD curve
       
for j=1:L % iteration over learning curves
   w = zeros(M,1);
   nw = zeros(1,N);
   
   for i=1:N  
     nw(i) = norm(wo-w)^2;
     u = randn(1,M)*sqR; % regressor
     d = u*wo + sqn*randn; % reference
     error = d - u*w; % error
     w = w + mu*u'*error;  % LMS recursion
   end
        
   msd_curve_sim = msd_curve_sim + nw;
end

msd_curve_sim  = msd_curve_sim/L;

msd_value  = mean(msd_curve_sim(N-200:N))

a=0;
b=0;
for i=1:M
  a = a + 1/(2-mu*lambda(i))  ;
  b = b + lambda(i)/(2-mu*lambda(i))  ;
end

  
msd_thy = mu*var_n*a/(1-mu*b)


figure
plot(1:N,10*log10(msd_curve_sim),'r',1:N,10*log10(msd),'b-')
xlabel('i');
ylabel('MSD (dB)');
legend('Simulation','Theory');
axis tight
grid

%%%%%%%%%%%%%%%%% GENERATE  MSD x STEP-SIZE PLOT

w  = zeros(M,1); % initial weight estimate
u  = zeros(1,M); % initial regressor

mu_vec = [0.03 0.05 0.07 0.08 0.09 0.095 0.1 0.125];
K = length(mu_vec);

aux = msd_value;
msd_value = zeros(1,K);
msd_value(1) = aux;
aux = msd_thy;
msd_thy = zeros(1,K);
msd_thy(1) = aux;

for kk=2:K
   
  mu = mu_vec(kk)
  msd_curve_sim = zeros(1,N); % simulated MSD curve
       
  for j=1:L % iteration over learning curves
    w = zeros(M,1);
    nw = zeros(1,N);
   
    for i=1:N  
      nw(i) = norm(wo-w)^2;
      u = randn(1,M)*sqR; % regressor
      d = u*wo + sqn*randn; % reference
      error = d - u*w; % error
      w = w + mu*u'*error;  % LMS recursion
    end
        
    msd_curve_sim = msd_curve_sim + nw;
  end

  msd_curve_sim  = msd_curve_sim/L;
  msd_value(kk)  = mean(msd_curve_sim(N-200:N))
  a=0;
  b=0;
  for i=1:M
    a = a + 1/(2-mu*lambda(i));
    b = b + lambda(i)/(2-mu*lambda(i));
  end
  msd_thy(kk) = mu*var_n*a/(1-mu*b)
end


for kk=1:K
    f(kk)=0;
    for i=1:M
     f(kk) = f(kk) + mu_vec(kk)*lambda(i)/(1-mu_vec(kk)*lambda(i));
    end
    f(kk) = (1/2)*f(kk);
end


figure
subplot(221)
plot(mu_vec,10*log10(msd_value),'ro-',mu_vec,10*log10(msd_thy),'bd-');
xlabel('\mu')
ylabel('MSD')
axis tight
grid
legend('Simulation','Theory')

subplot(222)
plot(mu_vec,f);
xlabel('\mu')
ylabel('f(\eta)')
axis tight
grid

