
% Solves part D
% Transient performance of LMS with non-Gaussian regressors
% A. H. Sayed
% March 2002

 close all;
 clear all;

 N = 10000;  % number of iterations
 
 M = 5; % number of taps
 wo = randn(M,1);
 wo = wo/norm(wo);

 var_un = 0.001; % uniform noise with variance 0.001
 squn = sqrt(var_un*12);
 
 mu = 0.05;

 rho = 5;  % desired eigenvalue spread
 lambda_min = 1;   % generate positive eigenvalues in the interval
 lambda_max = rho; % [lambda_min; lambda_max];
 eigenv = lambda_min + (lambda_max-lambda_min)*rand(M-2,1);
 R = diag([lambda_min;eigenv;lambda_max]); % Covariance matrix; it is diagonal = Lambda
 sqR = sqrt(R);
 
 lambda = [lambda_min;eigenv;lambda_max]';
 
 L = 30;

 w  = zeros(M,1); % initial weight estimate
 u  = zeros(1,M); % initial regressor

      
msd_curve_sim = zeros(1,N); % simulated MSD curve     
B = zeros(M^2,M^2);
A = zeros(M^2,M^2);
I = eye(M,M);
Re = zeros(M,M);

%%%%%%%%%%%%%%%%% GENERATE ENSEMBLE-AVERAGE MSD CURVE AND ESTIMATE MOMENTS

for j=1:L % iteration over learning curves
   j 
   w = zeros(M,1);
   nw = zeros(1,N);
   
   for i=1:N  
     nw(i) = norm(wo-w)^2;
     u = sqrt(12)*(rand(1,M)-0.5)*sqR; % regressor/ uniform distribution with variance R
     d = u*wo + squn*(rand-0.5); % reference with uniform noise
     error = d - u*w; % error
     w = w + mu*u'*error;  % LMS recursion
     B = B + kron((u'*u),(u'*u));
     A = A + kron(I,(u'*u)) + kron((u'*u),I);
     Re  = Re + u'*u;  % not needed; just for verification purposes
   end
        
   msd_curve_sim = msd_curve_sim + nw;
end


msd_curve_sim  = msd_curve_sim/L;
msd_value  = mean(msd_curve_sim(N-200:N));


%%%%%%%%%%%%%%%%% GENERATE THEORETICAL MSD CURVE
Re = Re/(L*N);
B = B/(L*N);
A = A/(L*N);
F = eye(M^2,M^2) - mu*A + mu*mu*B;

A2 = kron(I,R) + kron(R',I);
norm(A-A2) % just to confirm that ensemble value is close to actual value, which is A2

R
Re
norm(R-Re) % just to confirm 

one_vec = vec(eye(M,M)); % vec(I)
lambda_vec = vec(R); % vec(R)

msd_thy = mu*mu*var_un*lambda_vec'*(inv(eye(M^2,M^2)-F))*one_vec;

[msd_value msd_thy mu]

wo_bar = wo;

msd = zeros(1,N);
msd(1) = norm(wo)^2; % time indexing starts at 1 in matlab!
Fa = eye(M^2,M^2);

for i=1:N-1
  aux = Fa*(eye(M^2,M^2)-F)*one_vec;
  W1 = vec(aux); % undo the vec operation to generate the weight matrix
  b = mu*mu*var_un*(lambda_vec'*Fa)*one_vec;
  c = wo_bar'*W1*wo_bar;    
  msd(i+1) = msd(i) + b - c; % theoretical MSD curve for LMS with non-Gaussian regressors
  Fa = F*Fa;
end

figure
plot(1:N,10*log10(msd_curve_sim),'r',1:N,10*log10(msd),'b-')
xlabel('i');
ylabel('MSD (dB)');
legend('Simulation','Theory');
axis tight
grid


flag = 0;
if flag == 1
    
%%%%%%%%%%%%%%%%% GENERATE MSD x STEP-SIZE PLOT

w  = zeros(M,1); % initial weight estimate
u  = zeros(1,M); % initial regressor

mu_vec = [0.03 0.05 0.07 0.08 0.09 0.095 0.1 0.125 0.13];
K = length(mu_vec);

aux = msd_value;
msd_value = zeros(1,K);
msd_value(1) = aux;
aux = msd_thy;
msd_thy = zeros(1,K);
msd_thy(1) = aux;

for kk=2:K
   
  mu = mu_vec(kk)
  msd_curve_sim = zeros(1,N); % simulated MSD curve
       
  for j=1:L % iteration over learning curves
    w = zeros(M,1);
    nw = zeros(1,N);
   
    for i=1:N  
      nw(i) = norm(wo-w)^2;
      u = sqrt(12)*(rand(1,M)-0.5)*sqR; % regressor/ uniform distribution with variance R
      d = u*wo + squn*(rand-0.5); % reference with uniform noise
      error = d - u*w; % error
      w = w + mu*u'*error;  % LMS recursion
    end
        
    msd_curve_sim = msd_curve_sim + nw;
  end

  msd_curve_sim  = msd_curve_sim/L;
  msd_value(kk)  = mean(msd_curve_sim(N-200:N))
  msd_thy(kk) = mu*mu*var_un*lambda_vec'*(inv(eye(M^2,M^2)-F))*one_vec
end



figure
subplot(211)
plot(mu_vec,10*log10(msd_value),'ro-',mu_vec,10*log10(msd_thy),'bd-');
xlabel('\mu')
ylabel('MSD')
axis tight
grid
legend('Simulation','Theory')

end % end of flag

1/max(eig(inv(A)*B))  % stability limit


H = [A/2 -B/2; eye(M^2,M^2) zeros(M^2,M^2)];

largest = 0;
lambda = eig(H);
for i=1:2*M^2
  if ((abs(imag(lambda(i))) <= 1e-6) & (real(lambda(i)) > 0))  % real positive eigenvalue
      largest = max(largest,real(lambda(i)));
  end
end

1/largest % stability limit

figure
subplot(211)
plot(real(lambda))
xlabel('Re(\lambda)');
axis tight
grid
subplot(212)
plot(imag(lambda))
xlabel('Im(\lambda)');
axis tight
grid
