% OFDM Receiver
% Part A

clear all
close all

%Parameters
N = 64; 	% Number of OFDM tones (order of the FFT)
P = 16; 	% Cyclic prefix length
M = 8; 	    % Channel length
num_ch = 1;	% Number of channels simulated for BER curves
snr = [10];    % SNR
len_p = 8; 	% Length of each packet sent; i.e., number of symbols
QAM = 2; 		% Number of constellation points is 2^QAM
p_ind = [4:8:63];		% indices of the tones used as pilot tones
d_ind = setdiff([1:N],p_ind);	% indices of the tones used as data tones
f = fft(eye(N))/sqrt(N); % DFT matrix

%h is the complex channel taps
 h = (randn(1,M)+sqrt(-1)*randn(1,M))/sqrt(2*M); % E||h||^2 = 1
 h_p = [h zeros(1,N-M)];

%H is the circulant channel matrix
 H = zeros(0,0);
 H = h_p ;
 for c=2:N
   h_p_shift=h_p([N-c+2:N,1:N-c+1]);
   H = [H; h_p_shift];
 end 

 r=1;
 b = round(rand((N-length(p_ind))*len_p,QAM));
 s = bin2qam(b,QAM);
 x(d_ind,:) = reshape(s,N-length(p_ind),len_p);
 x(p_ind,:) = (1+sqrt(-1))/sqrt(2)*ones(length(p_ind),len_p);
 x_f = inv(f)*x;
 v = (randn(N+P-M,len_p)+sqrt(-1)*randn(N+P-M,len_p))/sqrt(2); % unit variance
 y = H*x_f+sqrt(10^(-snr(r)/10))*v(1:N,:);
 y_f = f*y;

 Qa = inv(f);
 Qa = Qa(1:N,1:M);
 Qb = inv(f);
 Qb = Qb(p_ind,1:M);

 %Channel estimate when all tones of the first symbol are used for training
 %Structure (a)

  h_hata(:,1) = inv(Qa'*Qa)*Qa'*((y_f(1:N,1)).*(conj(x(1:N,1))))/sqrt(N);

 %Channel estimate when multiple symbols are used for training 
 %Structure (b)

  for i=1:len_p
    h_hatb(:,i) = inv(Qb'*Qb)*Qb'*((y_f(p_ind,i)).*(conj(x(p_ind,i))))/sqrt(N);
  end

  h_hatb = mean(h_hatb,2);

  figure
  subplot(3,1,1)
  stem(abs(h))
  grid on
  title('(a)')

  subplot(3,1,2)
  stem(abs(h_hata))
  grid on
  title('(b)')

  subplot(3,1,3)
  stem(abs(h_hatb))
  grid on
  title('(c)')
  xlabel('channel tap index')

         

  figure
  subplot(3,1,1)
  stem(abs(inv(f)*h_p.'))
  grid on
  title('(a)')

  subplot(3,1,2)
  stem(abs(inv(f)*[h_hata; zeros(N-M,1)]))         
  grid on  
  title('(b)')

  subplot(3,1,3)
  stem(abs(inv(f)*[h_hatb; zeros(N-M,1)]))         
  grid on
  xlabel('tone index')
  title('(c)')
