% Tracking of a Rayleigh fading channel
% Solves part B

% A. H. Sayed
% Sep. 2002.

close all
clear all

var_n = 1e-3;
sqn = sqrt(var_n);

Ts = 0.8e-6; % sampling frequency is 1.25MHz
Ns = 1000;
fd = 80; % Doppler frequency
N = Ns;

epsilon = 1e-6;
lambda = 0.995;

M = 5;
channel  = zeros(M,N);
w_nlms_t = zeros(M,N);
w_rls_t  = zeros(M,N);
w_erls_t = zeros(M,N);

for i=1:M;
 channel(i,:) = rayleigh(Ts,Ns,fd);
end

alpha = bessel(0,2*pi*fd*Ts); 
q = 1-alpha*alpha;

[alpha q]

learn_nlms = zeros(1,N);
learn_rls = zeros(1,N);
learn_erls = zeros(1,N);

msd_nlms = zeros(1,N);
msd_rls = zeros(1,N);
msd_erls = zeros(1,N);

mu = 0.25;
L = 200;

expu = 0;


for k = 1:L
 k   
 
 input = randn(1,N);
 v = sqn*randn(1,N); 

 e = zeros(1,N);
 e_rls = zeros(1,N);
 e_erls = zeros(1,N);
 
 e1 = zeros(1,N);
 e1_rls = zeros(1,N);
 e1_erls = zeros(1,N);

 u = zeros(1,M);

 w = zeros(M,1);
 w_rls = zeros(M,1);
 w_erls = zeros(M,1);
 
 P_rls = (1/epsilon)*eye(M,M);
 P_erls = (1/epsilon)*eye(M,M);

 
   for j=1:N
     wo = channel(:,j);
     u = [input(j) u(1:M-1)];
     d = u*wo + v(j);

     factor = epsilon + norm(u)^2;
     e(j) = d - u*w;
     w = w + mu*u'*e(j)/factor;  % NLMS
     expu = expu + 1/factor;
            
     e_rls(j) = d - u*w_rls; 
     gamma = 1/(1 +(1/lambda)*u*P_rls*u'); 
     g = (1/lambda)*P_rls*u'*gamma; 
     w_rls = w_rls + g*e_rls(j);  % RLS 
     P_rls = (1/lambda)*P_rls - (g*g'/gamma);
    
     e_erls(j) = d - u*w_erls; 
     gamma = 1/(1 +(1/lambda)*u*P_erls*u'); 
     g = (1/lambda)*(alpha)*P_erls*u'*gamma; 
     w_erls = alpha*w_erls + g*e_erls(j);  % extended RLS 
     P_erls = q*eye(M,M) + (1/lambda)*(alpha^2)*P_erls - (g*g'/gamma);
    
     e1(j) = norm(w-wo)^2;
     e1_rls(j) = norm(w_rls-wo)^2;
     e1_erls(j) = norm(w_erls-wo)^2;
     
     w_nlms_t(:,j) = w;
     w_rls_t(:,j) = w_rls;
     w_erls_t(:,j) = w_erls;

   end
    learn_nlms = learn_nlms + abs(e).^2;
    learn_rls = learn_rls + abs(e_rls).^2;
    learn_erls = learn_erls + abs(e_erls).^2;

    msd_nlms = msd_nlms + abs(e1).^2;
    msd_rls = msd_rls + abs(e1_rls).^2;
    msd_erls = msd_erls + abs(e1_erls).^2;

end

learn_nlms = 10*log10(learn_nlms/L);
learn_rls = 10*log10(learn_rls/L);
learn_erls = 10*log10(learn_erls/L);

msd_nlms = 10*log10(msd_nlms/L);
msd_rls = 10*log10(msd_rls/L);
msd_erls = 10*log10(msd_erls/L);


expu = expu/(L*N)  % estimated value of E(1/norm(u)^2)


mse_nlms = mean(learn_nlms(N-100:N))
mse_rls = mean(learn_rls(N-100:N))
mse_erls = mean(learn_erls(N-100:N))

msd_ss_nlms = mean(msd_nlms(N-100:N))
msd_ss_rls = mean(msd_rls(N-100:N))
msd_ss_erls = mean(msd_erls(N-100:N))

%experimental values for MSE and MSD
[mse_nlms mse_rls mse_erls]
[msd_ss_nlms msd_ss_rls msd_ss_erls]

%theoretical values for MSE
R=eye(M,M);
Q=q*eye(M,M);
mse_nlms_thy = var_n + (mu*var_n*trace(R)*expu/(2-mu)) + ((1/mu)/(2-mu))*trace(R)*trace(Q);
mse_rls_thy = var_n + (var_n*(1-lambda)*M + (1/(1-lambda))*trace(Q*R))/(2-(1-lambda)*M);

[10*log10(mse_nlms_thy) 10*log10(mse_rls_thy)]


plot(1:N,learn_nlms,'b',1:N,learn_rls,'k',1:N,learn_erls,'r');
title('MSE')
xlabel('i')
ylabel('dB')
grid
axis tight
legend('NLMS','RLS','E-RLS')

figure
plot(1:N,msd_nlms,'b',1:N,msd_rls,'k',1:N,msd_erls,'r');
title('MSD')
xlabel('i')
ylabel('dB')
grid
axis tight
legend('NLMS','RLS','E-RLS')

figure
subplot(511)
plot(1:N,abs(channel(1,:)),'r',1:N,abs(w_nlms_t(1,:)),'b',1:N,abs(w_rls_t(1,:)),'g',1:N,abs(w_erls_t(1,:)),'m');
axis tight
grid
subplot(512)
plot(1:N,abs(channel(2,:)),'r',1:N,abs(w_nlms_t(2,:)),'b',1:N,abs(w_rls_t(2,:)),'g',1:N,abs(w_erls_t(2,:)),'m');
axis tight
grid

subplot(513)
plot(1:N,abs(channel(3,:)),'r',1:N,abs(w_nlms_t(3,:)),'b',1:N,abs(w_rls_t(3,:)),'g',1:N,abs(w_erls_t(3,:)),'m');
axis tight
grid

subplot(514)
plot(1:N,abs(channel(4,:)),'r',1:N,abs(w_nlms_t(4,:)),'b',1:N,abs(w_rls_t(4,:)),'g',1:N,abs(w_erls_t(4,:)),'m');
axis tight
grid

subplot(515)
plot(1:N,abs(channel(5,:)),'r',1:N,abs(w_nlms_t(5,:)),'b',1:N,abs(w_rls_t(5,:)),'g',1:N,abs(w_erls_t(5,:)),'m');
axis tight
grid

figure
plot(1:N,abs(channel(3,:)),'r',1:N,abs(w_nlms_t(3,:)),'b',1:N,abs(w_rls_t(3,:)),'g',1:N,abs(w_erls_t(3,:)),'m');
title('Magnitude of tap 3')
legend('channel','NLMS','RLS','ERLS')
axis tight
grid
