
% Active noise control
% Solves part D

% Ali H. Sayed
% Sep. 2002

clear all
close all

N = 5000;

load path;
M = max(size(path));

SNR = 60;
var_v = 10^(-SNR/10);
F = [1 -1.2 0.72];

alpha = 0.1;

epsilon = 1e-6;

L = 50; 

learn_lms = zeros(1,N);
learn_felms = zeros(1,N);
learn_fxlms = zeros(1,N);
learn_mfxlms = zeros(1,N);

for j=1:L
 j

 u = zeros(1,M);
 u1 = zeros(1,M);
 u2 = zeros(1,M);
 
 w_lms = zeros(M,1);
 w_felms = zeros(M,1);
 w_fxlms = zeros(M,1);
 w_mfxlms = zeros(M,1);

 e_lms = zeros(1,N);
 e_felms = zeros(1,N);
 e_fxlms = zeros(1,N);
 e_mfxlms = zeros(1,N);

 e1_felms = 0;
 e2_felms = 0;
 e1_fxlms = 0;
 e2_fxlms = 0;
 e1_mfxlms = 0;
 e2_mfxlms = 0;

 hatd1 = 0;
 hatd2 = 0;
 
 input = randn(1,N);
 d = filter(path,1,input) + sqrt(var_v)*randn(1,N);

 u = [input(1) u(1:M-1)];
 factor = (epsilon+norm(u)^2);   

 e_lms(1) = d(1) - u*w_lms;  % first iteration for NLMS
 w_lms = w_lms + (alpha/factor)*u'*e_lms(1); 

 e_felms(1) = d(1) - u*w_felms;   % first iteration for filtered-error LMS
 ef = F(1)*e_felms(1) + F(2)*e1_felms + F(3)*e2_felms;
 w_felms = w_felms + (alpha/factor)*u'*ef; 
    
 e_fxlms(1) = d(1) - u*w_fxlms;   % first iteration for filtered-x LMS
 ef = F(1)*e_fxlms(1) + F(2)*e1_fxlms + F(3)*e2_fxlms;
 uf = F(1)*u + F(2)*u1 + F(3)*u2;
 w_fxlms = w_fxlms + (alpha/factor)*uf'*ef; 


 hatd = u*w_mfxlms; 
 hatdf = F(1)*hatd + F(2)*hatd1 + F(3)*hatd2;
 e_mfxlms(1) = d(1) - hatd;   % first iteration for modified filtered-x LMS
 ef = F(1)*e_mfxlms(1) + F(2)*e1_mfxlms + F(3)*e2_mfxlms;
 w_mfxlms = w_mfxlms + (alpha/factor)*uf'*(ef - uf*w_mfxlms + hatdf); 

 for i=2:N
    u2 = u1;
    u1 = u;
    u = [input(i) u(1:M-1)];
    factor = (epsilon+norm(u)^2);

    e_lms(i) = d(i) - u*w_lms;  % NLMS
    w_lms = w_lms + (alpha/factor)*u'*e_lms(i); 

    e2_felms = e1_felms; 
    e1_felms = e_felms(i-1);
    e_felms(i) = d(i) - u*w_felms;   % filtered-error LMS
    ef = F(1)*e_felms(i) + F(2)*e1_felms + F(3)*e2_felms;
    w_felms = w_felms + (alpha/factor)*u'*ef; 
    
    e2_fxlms = e1_fxlms; 
    e1_fxlms = e_fxlms(i-1);
    e_fxlms(i) = d(i) - u*w_fxlms;   %  filtered-x LMS
    ef = F(1)*e_fxlms(i) + F(2)*e1_fxlms + F(3)*e2_fxlms;
    uf = F(1)*u + F(2)*u1 + F(3)*u2;
    w_fxlms = w_fxlms + (alpha/factor)*uf'*ef; 

    hatd2 = hatd1;
    hatd1 = hatd;
    hatd = u*w_mfxlms; 
    hatdf = F(1)*hatd + F(2)*hatd1 + F(3)*hatd2;
    e_mfxlms(i) = d(i) - hatd;   % modified filtered-x LMS
    ef = F(1)*e_mfxlms(i) + F(2)*e1_mfxlms + F(3)*e2_mfxlms;
    w_mfxlms = w_mfxlms + (alpha/factor)*uf'*(ef - uf*w_mfxlms + hatdf); 
    
   end
    learn_lms = learn_lms + abs(e_lms).^2;
    learn_felms = learn_felms + abs(e_felms).^2;
    learn_fxlms = learn_fxlms + abs(e_fxlms).^2;
    learn_mfxlms = learn_mfxlms + abs(e_mfxlms).^2;
  end  
  
  learn_lms = 10*log10(learn_lms/L);
  learn_felms = 10*log10(learn_felms/L);
  learn_fxlms = 10*log10(learn_fxlms/L);
  learn_mfxlms = 10*log10(learn_mfxlms/L);


figure
plot(1:N,learn_lms,'r',1:N,learn_felms,'b',1:N,learn_fxlms,'m',1:N,learn_mfxlms,'k');
legend('NLMS','FeLMS','FxLMS','mFxLMS');
axis tight
grid
ylabel('dB')
xlabel('i')
